@@ -2,10 +2,12 @@ package WebService::NetSuite::SuiteTalk;
 
 use Modern::Perl '2011';    ## no critic (Modules::ProhibitUseQuotedVersion)
 
-our $VERSION = '0.002';     # TRIAL VERSION
+our $VERSION = '0.003';     # TRIAL VERSION
 use utf8;
+use List::Util 1.33;
 use Moo;
 use MooX::Types::MooseLike::Base qw(CodeRef HashRef);
+use POSIX 'floor';
 with 'WebService::NetSuite::SuiteTalk::Role::Connection';
 
 has _calls => ( is => 'ro', isa => HashRef [CodeRef], default => sub { {} } );
@@ -22,15 +24,38 @@ sub _call {
     }
 
     # make the call and either return or throw an exception
-    my ( $answer, $trace ) = $self->_calls->{$operation_name}->(
-        passport   => $self->passport,
-        parameters => $parameters_ref,
-        @call_options,
-    );
-    if ( not $answer ) {
+    my @results;
+    push @results => [
+        $self->_calls->{$operation_name}->(
+            passport   => $self->passport,
+            parameters => $parameters_ref,
+            @call_options,
+        ),
+    ];
+    my ( $answer_ref, $trace ) = @{ $results[-1] };
+    if ( not $answer_ref ) {
         for ( $trace->errors ) { $_->throw }
     }
-    return ( $answer, $trace );
+
+    ## no critic (ValuesAndExpressions::ProhibitAccessOfPrivateData)
+    if ( my $faults_ref = $answer_ref->{Fault} ) {
+        my $fault_name
+            = $answer_ref->{ $faults_ref->{_NAME} }{error}{class}[2];
+        if ((   List::Util::any { $fault_name eq $_ }
+                qw(ExceededRecordCountFault ExceededRequestSizeFault)
+            )
+            and $operation_name =~ / List $/xms
+            and exists $parameters_ref->{record}
+            and 'ARRAY' eq ref $parameters_ref->{record}
+            and @{ $parameters_ref->{record} } > 1
+            )
+        {
+            push @results => map { $self->_call( @{$_} ) }
+                _halve_array( @{ $parameters_ref->{record} } );
+        }
+    }
+
+    return ( @results > 1 ? @results : @{ $results[0] } );
 }
 
 sub BUILD {
@@ -48,6 +73,13 @@ sub BUILD {
     return;
 }
 
+## no critic (Subroutines::RequireArgUnpacking)
+sub _halve_array {
+    my $item_split_idx = floor( $#_ / 2 );
+    return ( [ @_[ 0 .. $item_split_idx ] ],
+        [ @_[ ( $item_split_idx + 1 ) .. $#_ ] ] );
+}
+
 1;
 
 # ABSTRACT: NetSuite SOAP interface that caches compiled calls in memory
@@ -67,7 +99,7 @@ WebService::NetSuite::SuiteTalk - NetSuite SOAP interface that caches compiled c
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
@@ -96,6 +128,8 @@ L<XML::Compile::WSDL11|XML::Compile::WSDL11> to compile and execute that method
 against the specified NetSuite SuiteTalk service; subsequent calls can vary the
 parameters but will use the same compiled code.
 
+=head1 METHODS
+
 Available method names are dynamically loaded from the SuiteTalk WSDL file's
 operations, and can be passed either a hash or reference to a hash with the
 necessary parameters. In scalar context they return a reference to a hash
@@ -105,12 +139,25 @@ object suitable for debugging and exception handling.
 
 If there is no result then an exception will be thrown.
 
+=head2 Special note about list methods
+
+NetSuite's list methods (e.g., C<addList>, C<deleteList>, C<getList>,
+C<initializeList>, C<updateList>, C<upsertList>, and their asynchronous
+counterparts C<asyncAddList>, C<asyncDeleteList>, C<asyncGetList>,
+C<asyncInitializeList>, C<asyncUpdateList>, C<asyncUpsertList>) may all run
+into the various governance measures implemented to prevent web serviced from
+adversely affecting other users or the user experience. This module tries
+to mitigate this by detecting when the C<ExceededRecordCountFault> or
+C<ExceededRequestSizeFault> errors are triggered, and when detected splitting
+the record list in half and sending it as separate requests, repeating if
+necessary. Thus the results of list methods are returned as an array of
+answers and traces, rather than a simple paired answer and trace. This is
+reflected in the examples below.
+
 As of this writing the following operations are published in the NetSuite
 SuiteTalk API 2014.2 schema.  Consult NetSuite's Help Center for full
 documentation on input and output parameters for each operation.
 
-=head1 METHODS
-
 =head2 add
 
 Example:
@@ -142,7 +189,7 @@ Example:
 
     use Moo;
     with 'WebServices::NetSuite::SuiteTalk::Role::Country';
-    my $answer_ref = $suitetalk->addList( record => [
+    my $answer_ref = ( $suitetalk->addList( record => [
         {
             XSI_TYPE    => 'listRel:Customer',
             isPerson    => 1,
@@ -179,7 +226,7 @@ Example:
                 },
             ],
         },
-    ] );
+    ] ) )[0][0];
 
 =head2 asyncAddList
 
@@ -361,7 +408,7 @@ Example:
 
     use Moo;
     with 'WebServices::NetSuite::SuiteTalk::Role::Country';
-    my $answer_ref = $suitetalk->upsertList( record => [
+    my $answer_ref = ( $suitetalk->upsertList( record => [
         {
             XSI_TYPE    => 'listRel:Customer',
             isPerson    => 1,
@@ -398,7 +445,7 @@ Example:
                 },
             ],
         },
-    ] );
+    ] ) )[0][0];
 
 =for Pod::Coverage BUILD
 
